#include <Servo.h>

const int ServoPin = 8;
const int pauseButtonPin = 2;
const int startButtonPin = 3;
const int approachTrigPin = 4;
const int approachEchoPin = 5;
const int levelTrigPin = 6;
const int levelEchoPin = 7;
const int buzzerPin = 13;
const int rgbRedPin = 9;
const int rgbGreenPin = 10;
const int rgbBluePin = 11;
const int ledPin = 1;
const int lightSensorPin = A0;

Servo Servo1;

float trashLevel;
boolean pause;

void setup() {
  Servo1.attach(ServoPin);
  Servo1.write(90);                       //lid close
  pinMode(pauseButtonPin, OUTPUT);
  pinMode(startButtonPin, OUTPUT);
  pinMode(approachTrigPin, OUTPUT);
  pinMode(approachEchoPin, INPUT);  
  pinMode(levelTrigPin, OUTPUT);
  pinMode(levelEchoPin, INPUT);
  pinMode(buzzerPin, OUTPUT);
  pinMode(rgbRedPin, OUTPUT);
  pinMode(rgbGreenPin, OUTPUT);
  pinMode(rgbBluePin, OUTPUT);
  pinMode(ledPin, OUTPUT);
  pinMode(lightSensorPin, INPUT);
  pause = false;
}

void loop() {
  setLight();
  getButtonStatus();
  if (!pause){
    trashLevel = getDistance(levelTrigPin, levelEchoPin); //return a length in cm
    setLedIndicator();
    handleUser();   
  }
}

void setLight(){
  int lightLevel = analogRead(lightSensorPin);
  if (lightLevel > 980){
    digitalWrite(ledPin, HIGH);
  }
  else if (lightLevel < 900){      //give a 500-400=100 range of reading 
                                  //as buffer so that the light won't flash around 500
    digitalWrite(ledPin, LOW);
  }
}

void getButtonStatus(){
  if(digitalRead(pauseButtonPin)){  //pause button pressed
    pause = true;
    digitalWrite(rgbRedPin, LOW);
    digitalWrite(rgbGreenPin, LOW);
    digitalWrite(rgbBluePin, LOW);
    Servo1.write(0);                //lid open
  }
  if (digitalRead(startButtonPin)){ //start button pressed
    pause = false;
    Servo1.write(90);               //lid close
  }
}

float getDistance(int trigPin, int echoPin){
  digitalWrite(trigPin, LOW);       //clear off
  delayMicroseconds(5);
  digitalWrite(trigPin, HIGH);      // create a pulse to the trigger
  delayMicroseconds(10);            // the trigger pulse is 10 microsecond apart
  digitalWrite(trigPin, LOW);
  unsigned long duration = pulseIn(echoPin, HIGH); //measuring duration of pulse
  float cm = (duration/2.0)*0.034;  //float: calculation should have decimal place include integer
  return cm;  
}

void setLedIndicator(){
  if (trashLevel > 5){                           //  empty --> GREEN
    digitalWrite(rgbRedPin, LOW);
    digitalWrite(rgbGreenPin, HIGH);
    digitalWrite(rgbBluePin, LOW);
  }
  else if (trashLevel > 5 and trashLevel < 15){   //half full --> BLUE
    digitalWrite(rgbRedPin, LOW);
    digitalWrite(rgbGreenPin, LOW);
    digitalWrite(rgbBluePin, HIGH);
  }
  else {
    digitalWrite(rgbRedPin, HIGH);                // full --> RED
    digitalWrite(rgbGreenPin, LOW);
    digitalWrite(rgbBluePin, LOW);    
  }
}

void handleUser(){
  float userDistance = getDistance(approachTrigPin, approachEchoPin); //return a length in cm
  if (userDistance < 40 && userDistance > 0){
    if (trashLevel > 5){
      Servo1.write(0);
      setLight();
      delay(3000);
    }
    else {
      tone(buzzerPin,120, 500);
      delay(500);
    }
  }
  else {
    Servo1.write(90);
    delay(500);    
  }  
}
